c Subroutine to perform model selection on y_lam=x beta + epsi_delta
c using Monte Carlo Markov Chain techniques. Programmed by Michael Smith
c at the Australian Grad. School of Management, UNSW, Sydney
c v1 programmed 12/93; v2-3 1-8/94; Release v4.1 7-8/95 
c release v4.2 2/96

	subroutine bms4_2(n,p,txx,txy,tyy,indic,jay,
     +  it,Rone,Rzero,order,sens,indicprior,random,seed,lam,
     +  lnjacob,lamprior,trans,beta,vble,x,y,b,delta,
     +  outlier,deltaprior,deltamix,aa,bb,type)

	integer n,p,indic(*),jay(*),it,delta(2,*)
	integer order(*),lam,vble(*),type
	double precision seed,lnjacob(*),deltamix(*),b,x(n,*)
	double precision sens,indicprior(*),lamprior(*),aa(*),bb(*)
	double precision txy(9,*),txx(it,*),tyy(*),beta(*)
	double precision Rone(it,*),Rzero(it,*),y(*),deltaprior(*)
	logical random,trans,outlier

c indic:indicator variables (indic(i)=1 if beta(i)!=0)
c n:#of obns, p:# colms of design matrix x, jay:length of gibbs stages
c txx,txy,tyy are x'x, x'y, y'y 
c indicprior(i):prior for indic(i)=1 
c delta(1.) old vector of outlier indicators
c delta(2.) new vector of outlier indicators

c local variables

	integer i,j,k,gmaxp,q,l,sumdelta
	parameter(gmaxp=450)
	integer perm0(gmaxp),perm1(gmaxp),iwk(gmaxp)
	double precision work(gmaxp),tvect(gmaxp),betahat(gmaxp)
	double precision sse1(9),sse0(9),mixturelam(9),kk(gmaxp)
	double precision wk2(gmaxp)
	logical mix,gdbug1,gdbug2,gpflag,dump
	parameter(gdbug1=.false.,gdbug2=.false.,gpflag=.false.)
	parameter(dump=.false.)

	call standardisex(n,p,kk,x,it)
	call initialise(n,p,q,mix,indic,it,txx,Rone,Rzero,
     +  sse0,sse1,txy,lam,iwk,tvect,trans,perm1,work,tyy,perm0,
     +  x,y,delta,b,sens,aa,bb,type,deltamix,beta,mixturelam,gpflag)  

c	if(dump)then
c          open(unit=203,file='dump.out',form='formatted',
c     +    access='sequential',status='unknown')
c	endif

	do k=1,3

	  if (k .eq. 1) then 
c	    print*,'beginning warmup period of length',jay(1)
	  elseif(k.eq.2)then
c	    print*,'calculating marginal disbn of transformation'
c	    print*,'using sampling period of length',jay(2)
	  else
	    call clamarginal(mixturelam,lam,jay)
c	    print*,'calculating mixture estimates of coeffts.'
c	    print*,'using sampling period of length',jay(3)
	    mix=.true.
	    trans=.false.
	  endif

c main loop
	  do i=1,jay(k)
	
	    if(dump)then
	      sumdelta=0
	      do j=1,n
	        sumdelta=sumdelta+delta(1,j)
	      enddo
c	      write(unit=203,fmt=100) q,sumdelta
c100	      format(i5,2x,i5)
	    endif

            if((dmod(dble(i),10.d0).lt.0.09d0).and.((q.ne.0).and.
     +      (outlier.eqv..false.)))then
c If outlier=F compute a full cholesky every 10 iterations.
              call fullchol(Rone,Rzero,sse1,sse0,q,txx,txy,gpflag,
     +        lam,iwk,tvect,it,p,trans,perm1,work,tyy,indic,sens)
	    elseif((dmod(dble(i),100.d0).lt.0.009d0).and.((q.ne.0)
     +      .and.(outlier)))then
c If outlier=T then recompute txxtxy every 100 iterations.
	      call calctxxtxy(txx,x,b,n,it,p,delta,txy,aa,bb,y,type,
     +        lam,trans)
            endif

c generate p(indic|delta,y)
	    call gindiccint(n,p,seed,txy,txx,tyy,order,
     +      indic,indicprior,q,sse0,sse1,it,Rzero,Rone,perm1,perm0,
     +      sens,random,lam,lamprior,lnjacob,trans,tvect,work,gpflag)

	    if(outlier)then

c generate from p(delta|indic,y)
	    call gdelta2(n,txx,txy,delta,indic,p,q,sens,
     +      lnjacob,lamprior,deltaprior,lam,tyy,perm1,Rone,Rzero,
     +      sse1,sse0,iwk,it,trans,work,b,y,x,kk,deltamix,
     +      mix,seed,tvect,aa,bb,type,gpflag)
	    endif

	    if(trans.and.(k.eq.2))then
c calculate marginal p(lam|y,indic,delta). Do not sample.
              call cmixlam(sse1,lam,mixturelam,lnjacob,lamprior,n,
     +        seed)
	    endif

c if mix calculate mixture estimates beta

	    if(mix)then
	      call cintmixests(perm1,indic,beta,it,p,
     +        Rone,sse1,q,work,tvect,Rzero,n,txy,lam,sens,
     +        betahat)
	    endif

	    if(gdbug1)then
	      do l=1,p
c		print*,'l,indic(l),perm1(l)',l,indic(l),perm1(l)
	      enddo
c	      print*,'lam,tyy(lam),sse1(lam)',lam,tyy(lam),sse1(lam)
c	      print*,'sse0(lam)',sse0(lam)
	    endif

	  enddo

c	  print*,'finished.'
	enddo

c	if(dump)then
c          endfile(unit=203)
c          close(unit=203)
c        endif


c upon exit:	deltamix=mix. est. for delta.
c		lam=mode of mix. est. for lambda.
c		lamprior=mix. est. for lambda. 
c		beta=mix. est. for beta.	

        do i=1,n
          deltamix(i)=deltamix(i)/jay(3)
        enddo
	do i=1,9
	  lamprior(i)=mixturelam(i)/jay(2)
	enddo
        do i=1,p
	  if(kk(i).gt.0.d0)then
            beta(i)=beta(i)/(dble(jay(3))*kk(i))
	  else
	    beta(i)=0.d0
	  endif
        enddo
	if(gdbug2)then
	  do i=1,9
c            print*,'i,lamprior(i)',i,lamprior(i)
          enddo
          do i=1,p
c            print*,'i,beta(i),kk(i)',i,beta(i),kk(i)
          enddo
	endif
	call destandardisex(n,p,kk,x,txx,txy,it)

	return
	end
c________________________________________________________
	subroutine gindiccint(n,p,seed,txy,txx,tyy,order,
     +  indic,indicprior,q,sse0,sse1,it,Rzero,Rone,perm1,perm0,
     +  sens,random,lam,lamprior,lnjacob,trans,tvect,work,gpflag)

c routines called: permute2,gran,solvindlv,dchex

	integer n,order(*),indic(*),q,perm0(*),perm1(*),p
	integer it,lam
	double precision txy(9,*),txx(it,*),seed,tyy(*),lnjacob(*)
	double precision indicprior(*),Rone(it,*),Rzero(it,*)
	double precision sens,sse1(*),sse0(*),lamprior(*)
	double precision tvect(*),work(*)
	logical random,trans,gpflag

c local variables

	integer i,j,lmaxp,k,l,jj,oq
	parameter(lmaxp=450)
	double precision z(2,2),c(lmaxp),s(lmaxp) 
	double precision prob,l1
	double precision h,gran,lnh,small,overflow,dummy
	logical finish,psd,ldbug1,ldbug2
	parameter(small=1.d-13,ldbug1=.false.,ldbug2=.false.)
	parameter(overflow=700.d0)
c overflow is such that exp(overflow)=Inf and exp(-overflow)=0
c on the machine.

	if(random)then
          call permute2(order,p,seed)
	endif

	do k=1,p
	  psd=.false.
	  i=order(k)

	  if (indic(i).eq.0) then

c add row R(.,q+1) to cholesky Rone

	    do j=1,q
	      if(i.le.perm1(j))then
	        Rone(j,q+1)=txx(i,perm1(j))
	      else
	        Rone(j,q+1)=txx(perm1(j),i)
	      endif
	      do l=1,j-1
	 	Rone(j,q+1)=Rone(j,q+1)-Rone(l,q+1)*Rone(l,j)
	      enddo
	      Rone(j,q+1)=Rone(j,q+1)/Rone(j,j)
	    enddo
	    Rone(q+1,q+1)=txx(i,i)
	    do l=1,q
	      Rone(q+1,q+1)=Rone(q+1,q+1)-(Rone(l,q+1)**2)
	    enddo

c extra lines to check matrix psd. If so, then indic(i).eq.0

	    if (Rone(q+1,q+1).lt.small) then
	      psd=.true.
c	      if(gpflag)then
c	        print*,'matrix psd'
c	      endif
	    else
	      Rone(q+1,q+1)=dsqrt(Rone(q+1,q+1))
	    endif

	    if(.not.psd)then
	      perm1(q+1)=i

	      if(trans)then

c calculate sse1 for all 9 values of lambda
	        do l=1,9
c replaced 21/11/95 to remove O(p) calculation with O(q)
c	           do j=1,p
c	             work(j)=txy(l,j)
c	   	   enddo
c                  call solvindlv(Rone,work,tvect,it,q+1,perm1)
		  do j=1,q+1
	            work(j)=txy(l,perm1(j))
	          enddo
	          call solvlvdash(Rone,work,tvect,it,q+1)
                  sse1(l)=tyy(l)
	          do j=1,q+1
c                   sse1(l)=sse1(l)-sens*(tvect(j)**2)/(1.d0+sens)
		    sse1(l)=sse1(l)-tvect(j)**2
	          enddo
	        enddo

	      else
c calculate sse1 for lam only
c replaced 21/11/95 to remove O(p) calculation with O(q)
c	 	do j=1,p
c                  work(j)=txy(lam,j)
c		  if(ldbug1)then
c		    print*,'txy(lam,j),j',txy(lam,j),j
c		  endif
c                enddo
c                call solvindlv(Rone,work,tvect,it,q+1,perm1)

		do j=1,q+1
		  work(j)=txy(lam,perm1(j))
		enddo
		call solvlvdash(Rone,work,tvect,it,q+1)

                sse1(lam)=tyy(lam)
                do j=1,q+1
c                 sse1(lam)=sse1(lam)-sens*(tvect(j)**2)/(1.d0+sens)
		  sse1(lam)=sse1(lam)-(tvect(j)**2)
                enddo
c		if(ldbug1)then
c                  print*,'tyy(lam),lam',tyy(lam),lam
c		  print*,'sse1(lam)',sse1(lam)
c		endif
	      endif

	    endif

	  else 

c remove a colm from the cholesky.
c jj = colm of Rzero which corresponds to i.
c find jj s.t. perm0(jj)=i

	    finish = .false.
	    jj=1
	    do while ((finish.eqv..false.).and.(jj.le.q))
	      if (perm0(jj).eq.i) then
	        finish = .true.
	      else
	        jj=jj+1
	      endif
	    enddo

	    if (jj.lt.q) then

c circular shift colms of R from 1,...,q to 1,..,jj-1,jj+1,..,q,jj

	      call dchex(Rzero,it,q,jj,q,z,2,0,c,s,2)

c reset perm0

	      do l=jj,q-1
	        perm0(l)=perm0(l+1)
	      enddo
	    elseif(jj.eq.q+1)then
c	      print*,'jj.eq.q+1- fatal error'
c	      stop
	    endif

	    if(trans)then
c calculate sse0 for all 9 values of lambda
	      do l=1,9
c replaced 21/11/95 to reduce calc. from O(p) to O(q)
c	        do j=1,p
c	     	  work(j)=txy(l,j)
c	        enddo
c	        call solvindlv(Rzero,work,tvect,it,q-1,perm0)

		do j=1,q-1
		  work(j)=txy(l,perm0(j))
		enddo
		call solvlvdash(Rzero,work,tvect,it,q-1)

	        sse0(l)=tyy(l)
                do j=1,q-1
                  sse0(l)=sse0(l)-(tvect(j)**2)
	        enddo
	      enddo

	    else
c calculate sse0 for lam only
c replaced 21/11/95 to reduce calc. from O(p) to O(q)
c	      do j=1,p
c                work(j)=txy(lam,j)
c                if(ldbug1)then
c                  print*,'txy(lam,j),j',txy(lam,j),j
c                endif
c              enddo
c              call solvindlv(Rzero,work,tvect,it,q-1,perm0)

	      do j=1,q-1
		work(j)=txy(lam,perm0(j))
	      enddo
	      call solvlvdash(Rzero,work,tvect,it,q-1)

              sse0(lam)=tyy(lam)
              do j=1,q-1
c               sse0(lam)=sse0(lam)-sens*(tvect(j)**2)/(1.d0+sens)
		sse0(lam)=sse0(lam)-(tvect(j)**2)
              enddo
c	      if(ldbug1)then
c              print*,'sse0(lam),tyy(lam),lam',sse0(lam),tyy(lam),lam
c	      endif
	    endif

	  endif

	  if (indic(i).eq.0)then
	    oq=q
	  else
	    oq=q-1
	  endif

	  if(trans)then
c calculate the probability indic(i)=1 having integrated over
c lambda.
	    prob=0.d0
	    h=0.d0
            do j=1,9
	     l1=0.d0	
              do l=1,9
	        dummy=lnjacob(l)-lnjacob(j)+dlog(lamprior(l))-
     +          dlog(lamprior(j))+(dble(n)/2.d0)*(dlog(sse0(j))-
     +          dlog(sse1(l)))
c overflow/underflow check for converting back from log scale.
	        if(dummy.gt.overflow)then
	          l1=l1+dexp(overflow)
	 	elseif(dummy.lt.-overflow)then
	 	  l1=l1+dexp(-overflow)
		else
		  l1=l1+dexp(dummy)
	        endif
	      enddo
	      h=h+1.d0/l1
	    enddo
	    h=h*((1.d0-indicprior(i))/indicprior(i))*dsqrt(sens+1.d0)

	  else
c calculate the prob indic(i)=1 for fixed lambda=lam
            lnh=dlog(1.d0-indicprior(i))-dlog(indicprior(i))
            lnh=lnh+dlog(sens+1.d0)/2.d0
            lnh=lnh+(dlog(sse1(lam))-dlog(sse0(lam)))*(dble(n)/2.d0)
	    if(lnh.gt.overflow)then
              h=dexp(overflow)
	    elseif(lnh.lt.-overflow)then
	      h=dexp(-overflow)
	    else
	      h=dexp(lnh)
	    endif
	  endif
	  prob=1.d0/(1.d0+h)

	  if(ldbug2)then
c	    print*,'i,h,prob,indic(i)',i,h,prob,indic(i)
	  endif

c generate indicator variable and reset b's, det's, R's and perm's

          if ((gran(seed).le.prob).and.(.not.psd)) then
	    if(trans)then
	      do l=1,9
	        sse0(l)=sse1(l)
	      enddo
	    else
	      sse0(lam)=sse1(lam)
	    endif
	    if (indic(i) .eq. 0) then
	      q=q+1
	      perm0(q)=perm1(q)
	      do l=1,q
	        Rzero(l,q)=Rone(l,q)
	      enddo
	    else
	      do l=1,q
	        do j=1,l
	          Rzero(j,l)=Rone(j,l)
	        enddo
	        perm0(l)=perm1(l)
	      enddo
	    endif
	    indic(i)=1

          else

	    if(trans)then
	      do l=1,9
	        sse1(l)=sse0(l)
	      enddo
	    else
	      sse1(lam)=sse0(lam)
	    endif
	    if (indic(i) .eq. 1) then
	      q=q-1
	      do l=1,q
                do j=1,l
                  Rone(j,l)=Rzero(j,l)
                enddo
                perm1(l)=perm0(l)
              enddo
	    endif
	    indic(i)=0

c note that if orgindic=0 and R1=R0 no changes are needed

          endif

c go to next indicator variable.

        enddo

	return
	end	
c________________________________________________________
	subroutine cmixlam(sse1,lam,mixturelam,
     +  lnjacob,lamprior,n,seed)

c routines called: none.

	integer lam,n
	double precision sse1(*),mixturelam(*)
	double precision lnjacob(*),lamprior(*),seed
	double precision lnterm,overflow
	parameter(overflow=700.d0)

c local variables

	integer k,j
	double precision prob(9),term

c calculate probabilities

	do j=1,9
	  term=0.d0
	  do k=1,9
	    if(k.ne.j)then
	      lnterm=lnjacob(k)-lnjacob(j)+dlog(lamprior(k))
     +        -dlog(lamprior(j))+(dble(n)/2.d0)*(dlog(sse1(j))-
     +        dlog(sse1(k)))
c altered 28/10/95 for overflow/underflow calculations.
	      if(lnterm.gt.overflow)then
	        term=term+dexp(overflow)
	      elseif(lnterm.lt.-overflow)then
	        term=term+dexp(-overflow)
	      else
	        term=term+dexp(lnterm)
	      endif
	    endif
	  enddo
	  prob(j)=1.d0/(1.d0+term)
	enddo
	
	do j=1,9
	  mixturelam(j)=mixturelam(j)+prob(j)
	enddo

	return
	end
c__________________________________________________________________
	subroutine fullchol(Rone,Rzero,sse1,sse0,q,txx,txy,gpflag,
     +  lam,iwk,tvect,it,p,trans,perm1,work,tyy,indic,sens)

c calculates  Rone, Rzero, sse1, sse0 given txx, perm1, tyy, txy.
c If psd, then indic changed so that it no longer is so. If trans=T
c then all nine values for sse1 and sse0 are calculated.

c routines called: newdchdc, solvindlv

	integer iwk(*),q,lam,it,p,perm1(*),indic(*)
	double precision Rone(it,*),Rzero(it,*),txx(it,*) 
	double precision tvect(*),sse1(*),sse0(*),txy(9,*),tyy(*)
	double precision work(*),sens
	logical trans,gpflag

c local variables

	integer l,j,info
	logical ldbug

	ldbug=.false.

        do j=1,q
          do l=1,j
	    if(perm1(l).le.perm1(j))then
              Rone(l,j)=txx(perm1(l),perm1(j))
	    else
	      Rone(l,j)=txx(perm1(j),perm1(l))
	    endif
          enddo
        enddo

	if(ldbug)then
	  do l=1,p	
c	    print*,'perm1(l),txx(1,l)',perm1(l),txx(1,l)
	  enddo
	endif

        call newdchdc(Rone,it,q,tvect,iwk,0,info)
        if(info.ne.q)then
	  if(gpflag)then
c            print134,'update cholesky readjusting matrix to be psd.',
c     +      'q reduced from',q,'to',info
c134         format(a45,1x,a19,1x,i3,a3,i3)
	  endif
	  do j=info+1,q
	    indic(perm1(j))=0
	  enddo
	  q=info
        endif

	if(trans)then
c find sse for all nine possible transformations.
          do j=1,9
c solve for tvect, Rone'(tvect)=x'y~(j)=work
c replaced 21/11/95 to reduce calc from O(p) to O(q)
c            do l=1,p
c              work(l)=txy(j,l)
c            enddo
c            call solvindlv(Rone,work,tvect,it,q,perm1)

	    do l=1,q
	      work(l)=txy(j,perm1(l))
	    enddo
	    call solvlvdash(Rone,work,tvect,it,q)
		
c calculate sse's
            sse1(j)=tyy(j)
            do l=1,q
c             sse1(j)=sse1(j)-sens*(tvect(l)**2)/(1.d0+sens)
	      sse1(j)=sse1(j)-tvect(l)**2
            enddo
            sse0(j)=sse1(j)
          enddo
	else
c find sse for untransformed data
c replaced 21/11/95 to reduce calc from O(p) to O(q)
c	  do l=1,p
c            work(l)=txy(lam,l)
c          enddo
c          call solvindlv(Rone,work,tvect,it,q,perm1)

	  do l=1,q
            work(l)=txy(lam,perm1(l))
	  enddo
	  call solvlvdash(Rone,work,tvect,it,q)

c calculate sse's
          sse1(lam)=tyy(lam)
          do l=1,q
c           sse1(lam)=sse1(lam)-sens*(tvect(l)**2)/(1.d0+sens)
	    sse1(lam)=sse1(lam)-(tvect(l)**2)
          enddo
          sse0(lam)=sse1(lam)
	endif

c set Rzero=Rone
        do l=1,q
          do j=1,l
            Rzero(j,l)=Rone(j,l)
          enddo
        enddo

	return
	end
c________________________________________________________
	subroutine cintmixests(perm1,indic,beta,it,p,
     +  Rone,sse1,q,work,tvect,Rzero,n,txy,lam,sens,
     +  betahat)

c routines called: calcbetahat

	integer it,p,perm1(*),indic(*),q,n,lam
	double precision beta(*),betahat(*),sens
	double precision sse1(*),Rone(it,*),work(*),tvect(*)
	double precision Rzero(it,*),txy(9,*)

c local variables

	integer i

c calculate beta. Note that we descale this mixture estimate
c for beta at the end of the program.

	call calcbetahat(txy,betahat,lam,work,it,Rone,
     +  q,perm1,p)
c note here that we calculate the mixture estimate of beta under
c the diffuse prior on beta (ie sens=infinity)
c	betahat(i)=betahat(i)*sens/(1.d0+sens)
        do i=1,q
          beta(perm1(i))=beta(perm1(i))+betahat(i)
        enddo

	return
	end
c_______________________________________________________
	subroutine partialupdatexy(txx,txy,tyy,x,y,n,p,delta,it,b,
     +  i,aa,bb,type,lam,trans)

c updates txx,txy,tyy according to a change in a single delta given
c by i

c routines called: ahch

        integer n,p,delta(2,*),it,i,type,lam
        double precision x(n,*),y(*),tyy(*),txy(9,*),txx(it,*)
        double precision b,aa(*),bb(*),mout
	logical trans

c local variables

        integer j,k
	double precision ahch,f1,f2(9)

        if((delta(1,i).eq.0).and.(delta(2,i).eq.1))then
c if an obsn became an outlier then adjust txx, tyy, txy
c accordingly.
	  f1=(1.d0/b)-1.d0
	  if(trans)then
	    do k=1,9
	      f2(k)=(aa(k)+bb(k)*ahch(y(i),type,k))*f1
	    enddo
	  else
	    f2(lam)=(aa(lam)+bb(lam)*ahch(y(i),type,lam))*f1
	  endif
          do k=1,p
c mout introduced 22/11/95. Loop should be unrolled too.
	    mout=x(i,k)*f1
            do j=1,k
              txx(j,k)=txx(j,k)+x(i,j)*mout
            enddo
	    if(trans)then
	      do j=1,9
                txy(j,k)=txy(j,k)+x(i,k)*f2(j)
	      enddo
	    else
	      txy(lam,k)=txy(lam,k)+x(i,k)*f2(lam)
	    endif
          enddo
	  if(trans)then
	    do k=1,9
              tyy(k)=tyy(k)+(f2(k)**2)/f1
	    enddo
	  else
	    tyy(lam)=tyy(lam)+(f2(lam)**2)/f1
	  endif

c if an outlier became 'un'outlier'ed', alter txx,txy and tyy
c accordingly.

        elseif((delta(1,i).eq.1).and.(delta(2,i).eq.0))then
          f1=1.d0-(1.d0/b)
	  if(trans)then
            do k=1,9
              f2(k)=(aa(k)+bb(k)*ahch(y(i),type,k))*f1
            enddo
	  else
	    f2(lam)=(aa(lam)+bb(lam)*ahch(y(i),type,lam))*f1
	  endif
          do k=1,p
c mout introduced 22/11/95. Loop should be unrolled too.
	    mout=x(i,k)*f1
            do j=1,k
              txx(j,k)=txx(j,k)+x(i,j)*mout
            enddo
	    if(trans)then
              do j=1,9
                txy(j,k)=txy(j,k)+x(i,k)*f2(j)
              enddo
	    else
	      txy(lam,k)=txy(lam,k)+x(i,k)*f2(lam)
	    endif
          enddo
	  if(trans)then
            do k=1,9
              tyy(k)=tyy(k)+(f2(k)**2)/f1
            enddo
	  else
	    tyy(lam)=tyy(lam)+(f2(lam)**2)/f1
	  endif
        endif

        return
        end
c_____________________________________________________________
	subroutine updatexy(txx,txy,tyy,x,y,n,p,delta,it,b,aa,
     +  bb,type,lam,trans)

c Recalculate txx, txy, tyy based on any changes from old delta(1,.)
c to new delta(2,.) for all seven values of lambda.

c routines called: partialupdatexy

	integer n,p,delta(2,*),it,type,lam
	double precision x(n,*),y(*),tyy(*),txy(9,*),txx(it,*)
	double precision b,aa(*),bb(*)
	logical trans

c local variables

	integer i

        do i=1,n
	  call partialupdatexy(txx,txy,tyy,x,y,n,p,delta,it,b,
     +    i,aa,bb,type,lam,trans)
	enddo

	return
	end
c___________________________________________________________________
	subroutine calcbetahat(txy,betahat,lam,work,it,Rone,
     +  q,perm1,p)

c routine calculates betahat from the decomposition
c Rone'Rone=txx given txy and given lam. Note this is the betahat for
c the scaled version of x.

c routines called: solvindlv,solvuv

	integer p,q,lam,perm1(*)
	double precision txy(9,*),betahat(*),work(*)
	double precision Rone(it,*)

c local variables

	integer i

        do i=1,p
          betahat(i)=txy(lam,i)
        enddo
        call solvindlv(Rone,betahat,work,it,q,perm1)
        call solvuv(Rone,work,betahat,it,q,1)

	return
	end
c__________________________________________________________________
	subroutine calctxxtxy(txx,x,b,n,it,p,delta,txy,aa,bb,y,type,
     +  lam,trans)

c routines called: ahch

	integer n,it,p,delta(2,*),type,lam
	double precision txx(it,*),b,x(n,*),txy(9,*),aa(*),bb(*)
	double precision y(*)
	logical trans

c local variables

	integer i,j,k,md,mdi
	double precision ahch,mout,mout2

c calculate integers for unrolled loop.
        md=mod(n,8)
        mdi=md+1

	do j=1,p
	  do i=1,j
c mout introduced 20/11/95. Loop unrolled 21/11/95 .
	    mout=0.d0
c	    do k=1,n
c	      if(delta(2,k).eq.1)then
c	        mout=mout+x(k,j)*x(k,i)/b
c	      else
c	        mout=mout+x(k,j)*x(k,i)
c	      endif
c	    enddo
	    do k=1,md
	      if(delta(2,k).eq.1)then
                mout=mout+x(k,j)*x(k,i)/b
              else
                mout=mout+x(k,j)*x(k,i)
              endif
	    enddo
	    do k=mdi,n,8
              if(delta(2,k).eq.1)then
                mout=mout+x(k,j)*x(k,i)/b
              else
                mout=mout+x(k,j)*x(k,i)
              endif
              if(delta(2,k+1).eq.1)then
                mout=mout+x(k+1,j)*x(k+1,i)/b
              else
                mout=mout+x(k+1,j)*x(k+1,i)
              endif
              if(delta(2,k+2).eq.1)then
                mout=mout+x(k+2,j)*x(k+2,i)/b
              else
                mout=mout+x(k+2,j)*x(k+2,i)
              endif
              if(delta(2,k+3).eq.1)then
                mout=mout+x(k+3,j)*x(k+3,i)/b
              else
                mout=mout+x(k+3,j)*x(k+3,i)
              endif
              if(delta(2,k+4).eq.1)then
                mout=mout+x(k+4,j)*x(k+4,i)/b
              else
                mout=mout+x(k+4,j)*x(k+4,i)
              endif
              if(delta(2,k+5).eq.1)then
                mout=mout+x(k+5,j)*x(k+5,i)/b
              else
                mout=mout+x(k+5,j)*x(k+5,i)
              endif
              if(delta(2,k+6).eq.1)then
                mout=mout+x(k+6,j)*x(k+6,i)/b
              else
                mout=mout+x(k+6,j)*x(k+6,i)
              endif
              if(delta(2,k+7).eq.1)then
                mout=mout+x(k+7,j)*x(k+7,i)/b
              else
                mout=mout+x(k+7,j)*x(k+7,i)
              endif
	    enddo
	    txx(i,j)=mout
	  enddo
	  if(trans)then
	    do i=1,9
c mout introduced 20/11/95. inner loop unrolled 21/11/95.
	      mout=0.d0
c	      do k=1,n
c	        if(delta(2,k).eq.1)then
c		   mout=mout+x(k,j)*(aa(i)+bb(i)*
c     +             ahch(y(k),type,i))/b
c                else
c                   mout=mout+x(k,j)*(aa(i)+bb(i)*
c     +             ahch(y(k),type,i))
c	        endif
c	      enddo
              do k=1,md
                if(delta(2,k).eq.1)then
                  mout=mout+x(k,j)*(aa(i)+bb(i)*
     +            ahch(y(k),type,i))/b
                else
                  mout=mout+x(k,j)*(aa(i)+bb(i)*
     +            ahch(y(k),type,i))
                endif
              enddo
	      do k=mdi,n,8
	        if(delta(2,k).eq.1)then
                  mout=mout+x(k,j)*(aa(i)+bb(i)*
     +            ahch(y(k),type,i))/b
                else
                  mout=mout+x(k,j)*(aa(i)+bb(i)*
     +            ahch(y(k),type,i))
                endif
                if(delta(2,k+1).eq.1)then
                  mout=mout+x(k+1,j)*(aa(i)+bb(i)*
     +            ahch(y(k+1),type,i))/b
                else
                  mout=mout+x(k+1,j)*(aa(i)+bb(i)*
     +            ahch(y(k+1),type,i))
                endif
                if(delta(2,k+2).eq.1)then
                  mout=mout+x(k+2,j)*(aa(i)+bb(i)*
     +            ahch(y(k+2),type,i))/b
                else
                  mout=mout+x(k+2,j)*(aa(i)+bb(i)*
     +            ahch(y(k+2),type,i))
                endif
                if(delta(2,k+3).eq.1)then
                  mout=mout+x(k+3,j)*(aa(i)+bb(i)*
     +            ahch(y(k+3),type,i))/b
                else
                  mout=mout+x(k+3,j)*(aa(i)+bb(i)*
     +            ahch(y(k+3),type,i))
                endif
                if(delta(2,k+4).eq.1)then
                  mout=mout+x(k+4,j)*(aa(i)+bb(i)*
     +            ahch(y(k+4),type,i))/b
                else
                  mout=mout+x(k+4,j)*(aa(i)+bb(i)*
     +            ahch(y(k+4),type,i))
                endif
                if(delta(2,k+5).eq.1)then
                  mout=mout+x(k+5,j)*(aa(i)+bb(i)*
     +            ahch(y(k+5),type,i))/b
                else
                  mout=mout+x(k+5,j)*(aa(i)+bb(i)*
     +            ahch(y(k+5),type,i))
                endif
                if(delta(2,k+6).eq.1)then
                  mout=mout+x(k+6,j)*(aa(i)+bb(i)*
     +            ahch(y(k+6),type,i))/b
                else
                  mout=mout+x(k+6,j)*(aa(i)+bb(i)*
     +            ahch(y(k+6),type,i))
                endif
                if(delta(2,k+7).eq.1)then
                  mout=mout+x(k+7,j)*(aa(i)+bb(i)*
     +            ahch(y(k+7),type,i))/b
                else
                  mout=mout+x(k+7,j)*(aa(i)+bb(i)*
     +            ahch(y(k+7),type,i))
                endif
	      enddo
	      txy(i,j)=mout
	    enddo
	  else
	    i=lam
c mout introduced 20/11/95. Loop unrolled 21/11/95.
	    mout=0.d0
c            do k=1,n
c              if(delta(2,k).eq.1)then
c	        mout=mout+x(k,j)*(aa(i)+bb(i)*
c     +          ahch(y(k),type,i))/b
c              else
c	        mout=mout+x(k,j)*(aa(i)+bb(i)*
c     +          ahch(y(k),type,i))
c              endif
c            enddo
            do k=1,md
              if(delta(2,k).eq.1)then
                mout=mout+x(k,j)*(aa(i)+bb(i)*
     +          ahch(y(k),type,i))/b
              else
                mout=mout+x(k,j)*(aa(i)+bb(i)*
     +          ahch(y(k),type,i))
              endif
            enddo
            do k=mdi,n,8
              if(delta(2,k).eq.1)then
                mout=mout+x(k,j)*(aa(i)+bb(i)*
     +          ahch(y(k),type,i))/b
              else
                mout=mout+x(k,j)*(aa(i)+bb(i)*
     +          ahch(y(k),type,i))
              endif
              if(delta(2,k+1).eq.1)then
                mout=mout+x(k+1,j)*(aa(i)+bb(i)*
     +          ahch(y(k+1),type,i))/b
              else
                mout=mout+x(k+1,j)*(aa(i)+bb(i)*
     +          ahch(y(k+1),type,i))
              endif
              if(delta(2,k+2).eq.1)then
                mout=mout+x(k+2,j)*(aa(i)+bb(i)*
     +          ahch(y(k+2),type,i))/b
              else
                mout=mout+x(k+2,j)*(aa(i)+bb(i)*
     +          ahch(y(k+2),type,i))
              endif
              if(delta(2,k+3).eq.1)then
                mout=mout+x(k+3,j)*(aa(i)+bb(i)*
     +          ahch(y(k+3),type,i))/b
              else
                mout=mout+x(k+3,j)*(aa(i)+bb(i)*
     +          ahch(y(k+3),type,i))
              endif
              if(delta(2,k+4).eq.1)then
                mout=mout+x(k+4,j)*(aa(i)+bb(i)*
     +          ahch(y(k+4),type,i))/b
              else
                mout=mout+x(k+4,j)*(aa(i)+bb(i)*
     +          ahch(y(k+4),type,i))
              endif
              if(delta(2,k+5).eq.1)then
                mout=mout+x(k+5,j)*(aa(i)+bb(i)*
     +          ahch(y(k+5),type,i))/b
              else
                mout=mout+x(k+5,j)*(aa(i)+bb(i)*
     +          ahch(y(k+5),type,i))
              endif
              if(delta(2,k+6).eq.1)then
                mout=mout+x(k+6,j)*(aa(i)+bb(i)*
     +          ahch(y(k+6),type,i))/b
              else
                mout=mout+x(k+6,j)*(aa(i)+bb(i)*
     +          ahch(y(k+6),type,i))
              endif
              if(delta(2,k+7).eq.1)then
                mout=mout+x(k+7,j)*(aa(i)+bb(i)*
     +          ahch(y(k+7),type,i))/b
              else
                mout=mout+x(k+7,j)*(aa(i)+bb(i)*
     +          ahch(y(k+7),type,i))
              endif
	    enddo
	    txy(i,j)=mout
	  endif
	enddo

	return
	end
c________________________________________________________
	subroutine clamarginal(mixturelam,lam,jay)

c 1) finds the maximiser of the marginal density. 2) fixes lam
c to this value.

c routines called: none

	integer lam,jay(*)
	double precision mixturelam(*)

c local variables

	integer i
	double precision maximum

	maximum=0.d0	
	do i=1,9
	  if(mixturelam(i).gt.maximum)then
	     lam=i
	     maximum=mixturelam(i)
	  endif
	enddo

	return
	end
c________________________________________________________
        subroutine standardisex(n,p,kk,x,it)

c routines called: none 

        integer p,n,it
        double precision x(n,*),kk(*)

c normalise x as x=xK^-1 by a diagonal
c matrix K=diag(kk1,...,kkp) where kkj=sqrt(sumoveri(x(i,j)^2))

c local variables

        integer i,j

        do j=1,p
          kk(j)=0.d0
          do i=1,n
            kk(j)=kk(j)+x(i,j)**2
          enddo
          kk(j)=dsqrt(kk(j))
        enddo

        do j=1,p
	  if(kk(j).gt.0.d0)then
	    do i=1,n
              x(i,j)=x(i,j)/kk(j)
            enddo	
	  endif
        enddo

        return
        end
c__________________________________________________________________
        subroutine destandardisex(n,p,kk,x,txx,txy,it)

c routines called: none

        integer p,n,it
        double precision txx(it,*),x(n,*),txy(9,*),kk(*)

c local variables
        integer i,j

        do i=1,p
          do j=1,9
            txy(j,i)=txy(j,i)*kk(i)
          enddo
          do j=1,i
            txx(j,i)=txx(j,i)*kk(i)*kk(j)
          enddo
        enddo

	do j=1,p
          do i=1,n
            x(i,j)=x(i,j)*kk(j)
          enddo
        enddo

        return
        end
c_____________________________________________________________
	subroutine initialise(n,p,q,mix,indic,it,txx,Rone,Rzero,
     +  sse0,sse1,txy,lam,iwk,tvect,trans,perm1,work,tyy,perm0,
     +  x,y,delta,b,sens,aa,bb,type,deltamix,beta,mixturelam,gpflag)

c routines called: ahch, calctxxtxy, fullchol 

	integer n,p,q,indic(*),it,lam,iwk(*),perm1(*),perm0(*)
	integer delta(2,*),type
	double precision txx(it,*),Rone(it,*),Rzero(it,*),aa(*),bb(*)
	double precision sse0(*),sse1(*),txy(9,*),tvect(*),deltamix(*)
	double precision work(*),tyy(*),x(n,*),y(*),b,sens,beta(*)
	double precision mixturelam(*)
	logical mix,trans,gpflag

c local variables
	integer i,j
	double precision ahch

        mix=.false.

c set all deltas to zero (ie: start with no outliers.)

        do i=1,n
          delta(1,i)=0
          delta(2,i)=0
        enddo

c set q, perm1 and perm0
	q=0
	do i=1,p
	  if(indic(i).eq.1)then
	    q=q+1
	    perm1(q)=i
	    perm0(q)=i
	  endif
	enddo

c calculate initial txx,txy,tyy
	call calctxxtxy(txx,x,b,n,it,p,delta,txy,aa,bb,y,type,
     +  lam,trans)
	if(trans)then
	  do j=1,9
	    tyy(j)=0.d0
	    do i=1,n	
	      tyy(j)=tyy(j)+(aa(j)+bb(j)*ahch(y(i),type,j))**2
	    enddo
	  enddo
	else
	  j=lam
	  tyy(j)=0.d0
          do i=1,n    
            tyy(j)=tyy(j)+(aa(j)+bb(j)*ahch(y(i),type,j))**2
          enddo
	endif

	call fullchol(Rone,Rzero,sse1,sse0,q,txx,txy,gpflag,
     +  lam,iwk,tvect,it,p,trans,perm1,work,tyy,indic,sens)

c fix mixture estimates to zero

	do i=1,n
	  deltamix(i)=0.d0
	enddo
	do i=1,p
	  beta(i)=0.d0
	enddo
	do i=1,9
	  mixturelam(i)=0.d0
	enddo

	return
	end
c_______________________________________________________
	subroutine gdelta2(n,txx,txy,delta,indic,p,q,sens,
     +  lnjacob,lamprior,deltaprior,lam,tyy,perm1,Rone,Rzero,
     +  sse1,sse0,iwk,it,trans,work,b,y,x,kk,deltamix,
     +  mix,seed,tvect,aa,bb,type,gpflag)

c generates delta(i)| delta(j.ne.i), y, indic
c delta(i)=1 then obsn i is considered an outlier, otherwise 
c delta(i)=0 and it is not.

c Note (2/96) the Rzero matrix is used as work space
c (Rone is actually used as itself) and Rzero
c is updated so that Rzero=Rone at the end of routine. sse1 and sse0
c are used though.

c routines called: ahch, gran, newdchdc, solvindlv, partialupdatexy

	integer indic(*),n,q,p,delta(2,*),lam,perm1(*)
	integer iwk(*),it,type
	double precision txx(it,*),txy(9,*),tyy(*)
	double precision lnjacob(*),lamprior(*),deltaprior(*)
	double precision sse1(*),sse0(*),Rone(it,*),x(n,*)
	double precision Rzero(it,*),work(*),y(*),b,kk(*)
	double precision deltamix(*),seed,tvect(*),aa(*)
	double precision sens,bb(*)
	logical trans,mix,gpflag

c local variable

	integer i,j,k,l,info
	double precision h,sum,lnsum,g(9),ahch,lnh,mmin,lwkj(9)
	double precision gran,prob,out,in,overflow,mout,lwkk(9)
	double precision xvect(450),zvect(450),sc1,sc2,sc3
c remove with global once routine tested.
	logical ldbug2,ldbug1,psd
	parameter(ldbug2=.false.,ldbug1=.false.,overflow=700.d0)

c set out and in
	out=1.d0-(1.d0/b)
	in=(1.d0/b)-1.d0

	do i=1,n

c set delta(1.) to the old value of delta and generate delta(2.)
	  delta(1,i)=delta(2,i)

c calculate adjustment to y.
	  if(trans)then
            do j=1,9
	      g(j)=aa(j)+bb(j)*ahch(y(i),type,j)
	    enddo
	  else
	    g(lam)=aa(lam)+bb(lam)*ahch(y(i),type,lam)
	  endif

c place x(i,.) into xvect(i,.)
	  do j=1,q
	    xvect(j)=x(i,perm1(j))
	  enddo

c solve for zvect, xvect=Rone'zvect          O(q^2)
	  call solvlvdash(Rone,xvect,zvect,it,q)
	  sc1=0.d0
	  do j=1,q
	    sc1=sc1+zvect(j)**2
	  enddo

c PART1: calculate sse0 if delta(1,i)=1
	  
	  if(delta(1,i).eq.1)then
	    if(trans)then
	      do j=1,9
                mout=g(j)*out

c set work=x'y~(j)+adjustment(i,j)		O(q)
                do l=1,q
                  work(l)=txy(j,perm1(l))+xvect(l)*mout
                enddo

c solve for tvect, work=Rone'tvect		O(q^2)
                call solvlvdash(Rone,work,tvect,it,q)

c calculate sse0
	        sc2=0.d0
	        sc3=0.d0
		do l=1,q
		  sc2=sc2+zvect(l)*tvect(l)
		  sc3=sc3+tvect(l)**2
	        enddo
	        sc2=sc2**2
		sse0(j)=tyy(j)+out*(g(j)**2)
		sse0(j)=sse0(j)-sc3+out*sc2/(1.d0+out*sc1)

              enddo

            else
c find sse for untransformed data
	      mout=out*g(lam)

c set work=x'y~(j)+adjustment(i,j)		O(q)
	      do l=1,q
	        work(l)=txy(lam,perm1(l))+mout*xvect(l)
	      enddo

c solve for tvect, work=Rone'tvect              O(q^2)
              call solvlvdash(Rone,work,tvect,it,q)

c calculate sse0 				O(q)
              sc2=0.d0
              sc3=0.d0
              do l=1,q
                sc2=sc2+zvect(l)*tvect(l)
                sc3=sc3+tvect(l)**2
              enddo
              sc2=sc2**2
              sse0(lam)=tyy(lam)+out*(g(lam)**2)
              sse0(lam)=sse0(lam)-sc3+out*sc2/(1.d0+out*sc1)

	    endif

c PART2: Calculate sse1 for case where delta(1,i)=0
          else

            if(trans)then
              do j=1,9
	        mmin=g(j)*in

c set work=x'y~(j)+adjustment(i,j)              O(q)
                do l=1,q
                  work(l)=txy(j,perm1(l))+xvect(l)*mmin
                enddo

c solve for tvect, work=Rone'tvect              O(q^2)
                call solvlvdash(Rone,work,tvect,it,q)

c calculate sse1
                sc2=0.d0
                sc3=0.d0
                do l=1,q
                  sc2=sc2+zvect(l)*tvect(l)
                  sc3=sc3+tvect(l)**2
                enddo
                sc2=sc2**2
                sse1(j)=tyy(j)+in*(g(j)**2)
                sse1(j)=sse1(j)-sc3+in*sc2/(1.d0+in*sc1)

              enddo
            else

              mmin=in*g(lam)

c set work=x'y~(j)+adjustment(i,j)              O(q)
              do l=1,q
                work(l)=txy(lam,perm1(l))+mmin*xvect(l)
              enddo

c solve for tvect, work=Rone'tvect              O(q^2)
              call solvlvdash(Rone,work,tvect,it,q)

c calculate sse1
              sc2=0.d0
              sc3=0.d0
              do l=1,q
                sc2=sc2+zvect(l)*tvect(l)
                sc3=sc3+tvect(l)**2
              enddo
              sc2=sc2**2
              sse1(lam)=tyy(lam)+in*(g(lam)**2)
              sse1(lam)=sse1(lam)-sc3+in*sc2/(1.d0+in*sc1)

            endif

          endif

c PART3: calculate prob(delta(2,i)=1)
	  if(trans)then

c bug in loop removed 11/2/96
	    h=0.d0	
c local work vectors introduced 21/11/95 to remove repeated calculations.
	    mout=dble(n)/2.d0
	    do j=1,9
	      lwkj(j)=-lnjacob(j)-dlog(lamprior(j))+mout*dlog(sse0(j))
	      lwkk(j)=lnjacob(j)+dlog(lamprior(j))-mout*dlog(sse1(j))
	    enddo
	    do j=1,9
	      sum=0.d0
	      do k=1,9
		lnsum=lwkk(k)+lwkj(j)
c adjusted 28/10/95 for overflow/underflow calculations.
	        if(lnsum.gt.overflow)then
		  sum=sum+dexp(overflow)
		elseif(lnsum.lt.-overflow)then
	          sum=sum+dexp(-overflow)
		else
	          sum=sum+dexp(lnsum)
		endif
	      enddo
	      h=h+1.d0/sum
	    enddo

	  else
	    lnh=(dble(n)/2.d0)*(dlog(sse1(lam))-dlog(sse0(lam)))
	    if(lnh.gt.overflow)then
	      h=dexp(overflow)
	    elseif(lnh.lt.-overflow)then
	      h=dexp(-overflow)
	    else
	      h=dexp(lnh)
	    endif
	  endif	    
	  
	  h=h*dsqrt(b)*((1.d0-deltaprior(i))/deltaprior(i))
	  prob=1.d0/(1.d0+h)

	  sc1=gran(seed)

c four cases:-
c 	(a) delta(1,i)=0 and sc1 > prob (no change)
c	(b) delta(1,i)=1 and sc1 < prob (no change)
c	(c) delta(1,i)=0 and sc1 < prob (change but with check)
c	(d) delta(1,i)=1 and sc1 > prob (change no check)

c case (a)
	  if((sc1.ge.prob).and.(delta(1,i).eq.0))then
	    delta(2,i)=0
	    if(trans)then
	      do j=1,9
	        sse1(j)=sse0(j)
	      enddo
	    else
              sse1(lam)=sse0(lam)
            endif

c case (b)
	  elseif((sc1.lt.prob).and.(delta(1,i).eq.1))then
            delta(2,i)=1
            if(trans)then
              do j=1,9
                sse0(j)=sse1(j)
              enddo
            else
              sse0(lam)=sse1(lam)
            endif

c case (c)
	  elseif((sc1.lt.prob).and.(delta(1,i).eq.0))then
c update cholesky with `downdating'	O(q^2)+O(q)
	    do j=1,q
	      work(j)=xvect(j)*dsqrt(out)
	    enddo
	    call newdchdd(Rone,it,q,work,Rzero,it,0,1.d0,1.d0,zvect,
     +      tvect,info)
	    if(info.eq.-1)then
	      psd=.true.
	    else
	      psd=.false.
	    endif
c if psd make no change in delta and set sse1=sse0
	    if(psd)then
	      prob=0.d0
	      delta(2,i)=0
              if(trans)then
                do j=1,9
                  sse1(j)=sse0(j)
                enddo
              else
                sse1(lam)=sse0(lam)
              endif
	    else
c set sse0=sse1 and update txx, txy etc.
	      delta(2,i)=1
              if(trans)then
                do j=1,9
                  sse0(j)=sse1(j)
	        enddo
              else
                sse0(lam)=sse1(lam)
              endif
              call partialupdatexy(txx,txy,tyy,x,y,n,p,delta,it,b,i,
     +        aa,bb,type,lam,trans)
	    endif

c case (d)
	  else
	    delta(2,i)=0
            do j=1,q
              work(j)=xvect(j)*dsqrt(out)
            enddo
	    call newdchud(Rone,it,q,work,Rzero,it,0,1.d0,1.d0,
     +      zvect,tvect)
	    if(trans)then
              do j=1,9
                sse1(j)=sse0(j)
              enddo
            else
              sse1(lam)=sse0(lam)
            endif
            call partialupdatexy(txx,txy,tyy,x,y,n,p,delta,it,b,i,
     +      aa,bb,type,lam,trans)
          endif

	  if(mix)then
	    deltamix(i)=deltamix(i)+prob
	  endif

c go around for next outlier variable.
	enddo

c update Rzero

        do k=1,q
          do j=1,k
            Rzero(j,k)=Rone(j,k)
          enddo
        enddo

	return
	end
c_____________________________________________________________
