.BG
.FN br
.TL
Bayesian Nonparametric Regression
.DN
Bayesian Nonparametric Regression Program (v4.2) Author: Michael Smith
Date: 3/96
.DN
Performs Additive Nonparametric regression.
.CS
br(x, xs, y, density=c(0), j=c(100,100,300), plotlength=400, 
outlier=F, b=100, deltaprior=c(0.05), trans=F, plotit=F, type=1,
seed=345986.028184573, indicprior=c(0.5), sens=1000)
.RA
.AG x
Vector (matrix) of (`p1', say) linearly modelled independent variable(s).
.AG xs
Vector (matrix) of (`p2', say) independent variable(s) modelled using
cubic regression splines. These should only be indpt. variables which
it is sensible to model with cubic regression splines. This can
preclude highly discrete indpt variables.
.AG y
Vector containing the dependent variable.
.OA
.AG density 
Vector of length p2 containing the number of obsns between each
potential knot for the independent variables entered in `xs'. Beware
making density too low as this can introduce so many knots that the
design matrix becomes highly colinear. Too high will mean that too few
(or no) potential knot sites will be introduced. By default, set to
`rep(max(4*p2,as.integer(n/35)),p2)' which should work well for all but
the most oscillatory functions.
.AG j
A vector of length 3. The first element is the number of iterations in
the warmup period. The second is the number of iterations used to estimate
the marginal distribution of the data transformation.
The third is the number of iterations used to estimate
the remaining mixture estimates.
.AG plotlength
A scalar giving the length of the grid over which the nonparametric function
estimates are calculated.
.AG outlier
A logical scalar set to true if a robust regression is required. Note that
setting `outlier=T' will substantially slow the estimator down, esp. if there
are a large number of obsns.
.AG b
A scalar giving the variance of the second normal mixture relative to the
first.
.AG deltaprior
A vector of length n giving the prior probability that each obsn is an outlier.
By default set to 0.05, rendering a prior outlier probability for each obsn
to be uniformly set to 5%.
.AG trans
Logical scalar indicating whether a transformation is to be estimated.
.AG plotit
Logical scalar indicating whether plots of the nonparametric estimates should
be ploted upon end of estimation. Must have graphic device open if `plotit=T'.
.AG type
Integer indicating which type of transformation is to be considered. 1 
is for the increasing power transformation (where`y' must be in the range
`(0,Inf)'
and 2 is for inverse link style transformations (where `y' must be
in the range (0,1))
.AG seed
Double precision scalar `seed' for the random number generators.
.AG indicprior
Real vector giving the prior probability that any the potential terms
of the regression splines are included. By default, set to be diffuse
at `rep(0.5,lots)' where `lots' = the number of terms in the regression spline.
To always include an intercept set `indicprior=c(1,rep(0.5,350))', to always
exclude an intercept set `indicprior=c(0,rep(0.5,350))'.
.AG sens
The diffuse scalar from the model selection framework in Smith and Kohn 
(1996). Note that this is not a smoothing parameter and its default should
be adequate for almost all situations encountered.
.RT
.AG xplot
A matrix of size `(plotlength,p2)' used for plotting the nonparametric function
estimates. (See test examples.)
.AG yplot
A matrix of size `(plotlength,p2)' used for plotting the nonparametric function
estimates. (See test examples.)
.AG resids
A vector of residuals from the regression.
.AG numk
Vector giving the number of knots successfully placed for each variable
entered in xs.
.AG fpres
A matrix of size `(n,p2)'. Each column `fpres[,i]'
corresponds to a vector of `f(xs[,i])+resids' and is used for diagnosing
a fit of an additive model.
.AG beta
A vector giving the mixture estimate of `beta'.
The first element is always the intercept estimate, while the next `p1' terms
are the estimated `betas' from the terms modelled linearly.
.AG tab
A table giving information about the transformation if `trans=T'.
The transformations of the form y |--> a + b.h(y) are considered, where
a and b are scalars and h(y) is either a power or inverse link transformation.
The estimate of the marginal distribution is given.
.AG delta
A vector of length `n' giving the posterior probability that
each obsn is an outlier.
.AG knot
A matrix giving the locations of the knots. Each row corresponds to a
different variable and the vector `knot[i,1:numk[i]]' gives the knot
locations for variable `i'.
.SH REFERENCES
Smith, M. and Kohn, R., "Nonparametric Regression using Bayesian Variable
Selection", To appear in 
.ul
Journal of Econometrics.
.EX
For examples, see the test programs accompanying this code package.
.WR
