c fortran header for fitting splines by bayesian model selection in an
c Includes two types of transformations (indexed by type) 9 levels
c in each. The tranformation is of the form y |--> aa + bb * ahch(y)

	subroutine breg4_2(n,p1,p2,xms,xspline,y,density,xplot,yplot,
     +  iwkn,indic,jay,sens,plotlength,numk,knot,
     +  sxs,txy,txx,it,resids,indicprior,seed,fpres,lam,
     +  lamprior,trans,z,beta,x,outlier,delta,deltaprior,deltamix,
     +  Rone,Rzero,type,wk1,wk2,aa,bb,random,b)

c routines called: ahch, plusop, dpvbubsort, constructx, calcjacob
c constructz, allocvarnames, bms4_2, cmixf,

	integer n,p1,p2,density(*),numk(*),indic(*),jay(*)
	integer plotlength,it,delta(2,*),iwkn(*),lam,type
	double precision xms(n,*),xspline(n,*),y(*),deltaprior(*)
	double precision xplot(plotlength,*),yplot(plotlength,*)
	double precision knot(p2,*),deltamix(*)
	double precision sens,sxs(*),resids(*),txx(it,*),txy(9,*)
	double precision lamprior(*),indicprior(*),seed,b
	double precision fpres(n,*),z(plotlength,p2,*),Rzero(it,*)
	double precision beta(*),aa(*),bb(*),x(n,*),Rone(it,*)
	double precision wk1(n,*),wk2(*)
	logical trans,outlier,random,dbug1
	parameter(dbug1=.false.)

c xplot the x grid for the mixture and modal estimates.
c yplot the corresponding modal estimates of y for xplot
c beta: beta for mixture estimate.
c resids(*): residuals from fit.

c local variables

	integer gmaxp,p
        parameter(gmaxp=450)
	integer g,h,i,j,k,l,jj
	integer vble(gmaxp),order(gmaxp)
	double precision tyy(9),lnjacob(9)
	double precision ahch,scale
	logical keepgoing, intercept

c	print*,'Nonparametric Bayesian Regression: v4.1'	
	intercept=.true.
	p=0
c	print*,'constructing matrices'
	if (intercept) then
	  p=1
	endif
	p=p+p1+3*p2

c Construct knots

	k=0
	do l=1,p2

c sort each colm of xspline into ascending order

	  do g=1,n
	    sxs(g)=xspline(g,l)
	  enddo

c sort and calculate scale

	  call getpos(sxs,n,iwkn)
	  call sort1(sxs,wk2,n,iwkn)
	  do g=1,n
	    sxs(g)=wk2(g)
	  enddo
	  scale=sxs(n)-sxs(1)

c as a sideline, can construct xplot(.,l)

	  do h=1,plotlength
	    xplot(h,l)=sxs(1)+(dble(h-1)*scale)/dble(plotlength)
	  enddo
	  keepgoing=.true.
	  j=0
	  jj=0
	  oldknot=sxs(1)
	  do while(keepgoing)
	    j=j+1
	    jj=jj+1
	    knot(l,jj)=(sxs(j*density(l)+1)+sxs(j*density(l)))/2.d0

c If knot is not equal to the old knot then include. This is
c overcome discrete observations in the data.

	    if ((knot(l,jj)-oldknot).ge.(scale*1.d-6))then
	      oldknot=knot(l,jj)
	    else
	      jj=jj-1
	    endif

	    if((j+2)*density(l).gt.n)then
	      keepgoing=.false.
	      numk(l)=jj
	      p=p+jj
	    endif
	  enddo
	 
	enddo 

	do i=1,p
	  order(i)=i
	enddo
	
	call constructx(x,xms,xspline,knot,numk,n,p1,p2,
     +  p,intercept)

c construct lnjacob for the transformation aa + bb * ahch(.)

	call calcjacob(lnjacob,n,y,type,wk1,wk2,aa,bb,trans,lam)
	if(dbug1)then
	  do i=1,9
c	    print*,'aa(i),bb(i),i',aa(i),bb(i),i
	  enddo
	endif

c construct z, where z  holds the vector of spline terms for each
c of the p2 spline variables at each point on plotlength.

	call constructz(numk,knot,z,plotlength,p2,xplot)

c allocate variable names to each indicator variable.

 	call allocvarnames(vble,p1,p2,numk,intercept)	

c having constructed matricies, perform model selection.

	call bms4_2(n,p,txx,txy,tyy,indic,jay,
     +  it,Rone,Rzero,order,sens,indicprior,random,seed,lam,
     +  lnjacob,lamprior,trans,beta,vble,x,y,b,delta,
     +  outlier,deltaprior,deltamix,aa,bb,type)

c calculate mixture estimates and standard deviations.

	call cmixf(beta,it,z,plotlength,p2,
     +  yplot,p1,vble,p)

c calculate resids, fpres (on transformed scale.)

	do i=1,n
	  resids(i)=ahch(y(i),type,lam)*bb(lam)+aa(lam)
	enddo  

	do j=1,p
	  do i=1,n
	    resids(i)=resids(i)-x(i,j)*beta(j)
	  enddo
	enddo

	if(intercept)then
	  k=1
	else
	  k=0
	endif
	k=k+p1

	if(p2.ge.1)then
	  do j=1,p2
	    do i=1,n
	      fpres(i,j)=resids(i)
	    enddo
	  enddo

	  do j=1,p2
	    do l=1,(3+numk(j))
	      k=k+1
	      do i=1,n
	        fpres(i,j)=fpres(i,j)+beta(k)*x(i,k)
	      enddo
	    enddo
	  enddo
	     
	endif

	return
	end	
c_______________________________________________________________
	subroutine constructz(numk,knot,z,plotlength,p2,xplot)

c routines called: plusop

	integer p2,numk(*),plotlength
	double precision knot(p2,*),z(plotlength,p2,*)
	double precision xplot(plotlength,*)

c local variables

	integer i,j,k
	double precision plusop

c loop around for each splined variable
	do i=1,p2

c loop around for each point of xplot
	  do j=1,plotlength

	    z(j,i,1)=xplot(j,i)
	    z(j,i,2)=xplot(j,i)**2
            z(j,i,3)=xplot(j,i)**3
	    do k=1,numk(i)
	      z(j,i,k+3)=plusop(xplot(j,i)-knot(i,k))**3
	    enddo

	  enddo
	enddo

	return
	end
c____________________________________________________________
	subroutine allocvarnames(vble,p1,p2,numk,intercept)

c routine allocates the variable number from which each indicator
c came from.

c routines called: none

	integer p1,p2,vble(*),numk(*)
	logical intercept

c local variables

	integer i,j,k

	k=1
	if(intercept)then
	  vble(k)=0
	  k=k+1
	endif

c for each of the colms associated with a linear term.
	do i=1,p1
	  vble(k)=i
	  k=k+1
	enddo

c for each of the spline variables.
	do i=1,p2
	  do j=1,(3+numk(i))
	    vble(k)=i+p1
	    k=k+1
	  enddo
	enddo

	return
	end
c___________________________________________________________
	subroutine cmixf(beta,it,z,plotlength,p2,
     +  yplot,p1,vble,p)

c routines called: none

	integer p2,it,plotlength,p1,vble(*),p
	double precision beta(*)
	double precision z(plotlength,p2,*),yplot(plotlength,*)

c local variables

	integer i,j,k

c first set the plots to zero.

	do i=1,plotlength
	  do k=1,p2
	    yplot(i,k)=0.d0
	  enddo
	enddo

	do i=1,plotlength
	  j=1
	  do k=1,p
c go through each variable and if it is a splined variable, add its
c contribution onto the mixture estimate for that variable.
	    if((vble(k)-p1).gt.0)then
	      yplot(i,vble(k)-p1)=yplot(i,vble(k)-p1)+z(i,
     +        vble(k)-p1,j)*beta(k)
	    endif
            if(vble(k+1).ne.vble(k))then
              j=1
            else
              j=j+1
            endif
	  enddo
	enddo

	return
	end
c_______________________________________________________________
	subroutine constructx(x,xms,xspline,knot,numk,n,p1,p2,
     +  p,intercept)

c routines called: plusop

	integer p,n,p1,p2,numk(*)
	double precision x(n,*),xms(n,*),xspline(n,*)
	double precision knot(p2,*)
	logical intercept

c local variables

	integer i,j,k,l
	double precision plusop

	k=0
	if(intercept)then
	  k=k+1
	  do i=1,n
	    x(i,k)=1.d0
	  enddo
	endif
	do j=1,p1
	  k=k+1
	  do i=1,n
	    x(i,k)=xms(i,j)
	  enddo
	enddo
	do j=1,p2
	  do i=1,n
	    x(i,k+1)=xspline(i,j)
	    x(i,k+2)=xspline(i,j)**2
	    x(i,k+3)=xspline(i,j)**3
	  enddo
	  k=k+3
	  do l=1,numk(j)
	    k=k+1
	    do i=1,n
	      x(i,k)=plusop(xspline(i,j)-knot(j,l))**3
	    enddo
	  enddo
	enddo
	
	if(k.ne.p)then
c	  print*,'k.ne.p in construct x'
c	  stop
	endif

	return
	end
c_______________________________________________________
